from rocolib.api.composables.graph.Graph import Graph as BaseGraph
from rocolib.api.composables.graph.Drawing import Drawing
from rocolib.api.composables.Composable import Composable, output, widget
from rocolib.utils.tabs import BeamTabs, BeamTabDecoration, BeamSlotDecoration
from rocolib.utils.display import meshfig, cutfig
from dash import dcc


class DecorationComposable(Composable, BaseGraph):
    def __init__(self):
        BaseGraph.__init__(self)

class GraphComposable(Composable, BaseGraph):
    def __init__(self):
        BaseGraph.__init__(self)
        self._d = None

    @property
    def d(self):
        if self._d is None:
            self._d = Drawing(self)
            self._d.transform(relative=(0,0))
        return self._d

    def append(self, g2, prefix2, **kwargs):
        if kwargs.get("mirror", False):
            g2.mirror()
        if kwargs.get("invert", False):
            g2.invertEdges()
        g2.prefix(prefix2)

        if kwargs.get("root", False):
            self.faces = g2.faces + self.faces
            self.edges = g2.edges + self.edges
        else:
            self.faces.extend(g2.faces)
            self.edges.extend(g2.edges)

    def makeOutput(self, name, outputs, filedir=None, widgets=False, **ka):
        self._thickness = ka.get("thickness", 0)
        self.tabify(ka.pop("tabFace", BeamTabs), ka.pop("tabDecoration", BeamTabDecoration),
                    ka.pop("slotFace", None), ka.pop("slotDecoration", BeamSlotDecoration), **ka)
        if "joint" in ka:
            self.jointify(**ka)
        self.place(transform3D=ka.pop("transform3D", None))

        return Composable.makeOutput(self, name, outputs, filedir, widgets, **ka)

    def to2D(self, fp, **ka):
        if ka.get("mode", "") in "printer laser animate".split():
            return self.d.toSVG(fp, mode=ka.get("mode"))
        else:
            return self.d.toDXF(fp, mode=ka.get("mode", "silhouette"))

    def to3D(self, fp, **ka):
        if ka.get('format') == "stl":
            self.fullmesh.save("roco-model", fh=fp)
        elif ka.get('format') == "png":
            meshfig(self.fullmesh, axes=False).write_image(fp)
        elif ka.get('format') == 'webots':
            from rocolib.utils.roco2sim.wbo_nodes import wboConverter
            fp.write(wboConverter(self.meshes))

    printOut    = output("printer.svg", "manual cut pattern folding printouts", mode="printer") (to2D)
    laserOut    = output("laser.svg", "laser cutter cut-and-fold pattern", mode="laser")        (to2D)
    cutOut      = output("silhouette.dxf", "Silhouette cut-and-fold pattern", mode="silhouette")(to2D)
    animOut     = output("animate.svg", "input for OrigamiSimulator", mode="animate")           (to2D)
    autoOut     = output("autofold.dxf", "autofolding pattern", mode="autofold")                (to2D)

    stlOut      = output("model.stl", "3D STL model", binary=True, format="stl")                (to3D)
    pngOut      = output("render.png", "3D render", binary=True, format="png")                  (to3D)
    wboOut      = output("webots.wbo", "Webots robot", format="webots")                         (to3D)

    @widget('display3D', "3D model")
    def display3D(self, fp, **ka):
        kwargs = {k:v for k,v in ka.items() if k in "color scale".split()}
        return dcc.Graph(figure = meshfig(self.fullmesh, **kwargs))

    @widget('display2D', "2D cut pattern")
    def display2D(self, fp, **ka):
        kwargs = {k:v for k,v in ka.items() if k in "scale axes highlight".split()}
        return dcc.Graph(figure = cutfig(self.getEdgeSet(), **kwargs))
