from rocolib.api.ports import Port
from rocolib.utils.utils import prefix as prefixString


class EdgePort(Port):
    def __init__(self, parent, graph, edges, params):
        Port.__init__(self, parent, {})
        if isinstance(edges, str) or not hasattr(edges, "__iter__"):
            edges = [edges]
        if isinstance(params, str) or not hasattr(params, "__iter__"):
            params = [params]
        self.graph = graph
        self.edges = edges
        self.params = params

    def getEdges(self):
        return self.edges

    def getParams(self):
        return self.params

    def update(self, newparent, oldcomposable, newcomposable, prefix):
        self.setParent(newparent)
        if oldcomposable == self.graph:
            self.edges = [prefixString(prefix, e) for e in self.edges]
            self.graph = newcomposable

    def canMate(self, otherPort):
        try:
            return len(self.getEdges()) == len(otherPort.getEdges())
        except AttributeError:
            return False

    def toString(self):
        return str(self.getEdges())

    def attachTo(self, toPort, **kwargs):
        if isinstance(toPort, EdgePort):
            label1 = self.getEdges()
            label2 = toPort.getEdges()

            numedges = len(label1)

            for i in range(numedges):
                newargs = {}
                for key, value in kwargs.items():
                    if isinstance(value, (list, tuple)):
                        newargs[key] = value[i]
                    else:
                        newargs[key] = value
                try:
                    self.graph.mergeEdge(label1[i], label2[numedges-i-1], **newargs)
                except AttributeError:
                    # Edge not found, skipping
                    pass
