from rocolib.api.components import FoldedComponent
from rocolib.api.composables.graph.Face import Rectangle
from rocolib.utils.tabs import BeamTabDecoration, BeamSlotDecoration

class SimpleRectBeam(FoldedComponent):
  def define(self):
    self.addParameter("length", 100, paramType="length")
    self.addParameter("width", 20, paramType="length")
    self.addParameter("depth", 50, paramType="length")

    self.addParameter("addTabs", True, valueType="bool", hidden=True)
    self.addParameter("phase", 0, values=[0,1,2,3], hidden=True)
    self.addParameter("fullTab", False, valueType="bool", hidden=True)

    for i in range(4):
      self.addEdgeInterface("topedge%d" % i, "r%d.e2" % i, ["width", "depth"][i % 2])
      self.addEdgeInterface("botedge%d" % i, "r%d.e0" % i, ["width", "depth"][i % 2])
      self.addFaceInterface("face%d" % i, "r%d" % i)
    self.addEdgeInterface("slotedge", "r3.e1", "length")
    self.addEdgeInterface("tabedge", "r0.e3", "length")

    self.addEdgeInterface("top", ["r%d.e2" % i for i in range(4)], ["width", "depth"])
    self.addEdgeInterface("bot", ["r%d.e0" % (3-i) for i in range(4)], ["depth", "width"])

  def assemble(self):
    nf = 4
    def wfn(i):
      if i % 2:
        return self.p.depth
      else:
        return self.p.width
    rs = [Rectangle("", wfn(i), self.p.length) for i in range(nf)]

    p = self.p.phase
    fromEdge = None
    for j in range(nf):
      i = (j + p) % nf
      self.attachEdge(fromEdge, rs[i], "e3", prefix="r%d"%i, angle=90)
      fromEdge = 'r%d.e1' % i
      self.setFaceInterface("face%d" % i, "r%d" % i)

    tabEdge, slotEdge = f"r{p}.e3", f"r{(p-1)%nf}.e1"

    if self.p.addTabs:
      mw = min(10, wfn(p-1))
      tab = Rectangle("", self.p.fullTab and wfn(p-1) or mw, self.p.length)
      self.attachEdge(tabEdge, tab, "e1", prefix="tab", angle=90)
      BeamTabDecoration(tab, "tab.e1", mw)
      BeamSlotDecoration(rs[(p-1)%nf], slotEdge, mw)
      tabEdge = "tab.e3"

    self.setEdgeInterface("slotedge", slotEdge, "length")
    self.setEdgeInterface("tabedge", tabEdge, "length")

if __name__ == "__main__":
  SimpleRectBeam.test()
