from rocolib.api.components import FoldedComponent
from rocolib.api.composables.graph.Face import RegularNGon2
from rocolib.api.composables.graph.Face import Rectangle
import numpy as np
import warnings

class Tire(FoldedComponent):
    """Tire Component
    Auto-select tire design based on thickness specified.
    If 0 or nothing specified, use a disk (regular ngon, 40)
    Else Nonagon tire with specified thickness.
    paramType=tire_thickness: control tire thickness
    paramType=radius: to set radius"""
    def define(self):
        self.addParameter("tire_thickness", 0, paramType="length")
        self.addParameter("radius", 25, paramType="length")
        self.addFaceInterface("face", "w1")

    def assemble(self):
        thickness = self.p.tire_thickness
        if thickness == 0:
            #tickness is 0 or not specified
            self.regularNgonTireAssembly()
        else:
            #if thickness is not zero
            self.foldedTireAssembly()

    def regularNgonTireAssembly(self):
        # number of polygon sides
        n = 40
        self.addFace(RegularNGon2("w1", n, self.p.radius))

    def foldedTireAssembly(self):
        # number of polygon sides
        n = 9
        r = self.p.radius
        tw = self.p.tire_thickness
        # length of
        sl = float(2 * r * np.cos(np.deg2rad(90-180/n)))

        wheel1 = RegularNGon2("", n, r)
        wheel2 = RegularNGon2("", n, r)

        support = []
        for i in range(n):
            support.append(Rectangle("", tw, sl))

        fromEdge = None
        for i in range(9):
            self.attachEdge(fromEdge, support[i], "e2", prefix="r%d" % i, angle=-40, root=(i == 1))
            fromEdge = 'r%d.e0' % i

        self.attachEdge("r0.e1", wheel1, "e0", prefix="w1", angle=-90)
        self.attachEdge("r4.e3", wheel2, "e0", prefix="w2", angle=-90)

        self.addTab("r8.e1", "w1.e8", angle=-90, width=11)
        self.addTab("r0.e3", "w2.e4", angle=-90, width=11)
        self.addTab("r4.e1", "w1.e4", angle=-90, width=11)

        self.addTab("r2.e1", "w1.e2", angle=-90, width=11)
        self.addTab("r2.e3", "w2.e2", angle=-90, width=11)

        self.addTab("r7.e1", "w1.e7", angle=-90, width=11)
        self.addTab("r7.e3", "w2.e6", angle=-90, width=11)


if __name__ == "__main__":
    Tire.test()
