from rocolib.api.components import FoldedComponent
from rocolib.api.composables.graph.Face import Rectangle, Face

class TwoDOF(FoldedComponent):
  def define(self):
    self.addParameter("mountlength", 40, paramType="length")
    self.addParameter("handlelength", 15, paramType="length")
    self.addParameter("hingelength", 10, paramType="length")
    self.addParameter("thickness", 10, paramType="length")
    self.addParameter("pitch", 15, paramType="angle", minValue=-90, maxValue=90)
    self.addParameter("yaw", 10, paramType="angle", minValue=-90, maxValue=90)

    self.addEdgeInterface("mountedge", "mount.e0", "mountlength")
    self.addEdgeInterface("outedge", "r2.e2", "handlelength")
    self.addEdgeInterface("handleedge", "r2.e3", "thickness")

  def assemble(self):
    t = self.p.thickness
    w = self.p.mountlength
    x = self.p.hingelength
    h = self.p.handlelength
    p = self.p.pitch
    y = self.p.yaw

    b = Face("", ((w, 0), (w, t), (w-x,t), (0,t), (0,0)))
    b0 = Rectangle("", w, 0)
    r1 = Rectangle("", x, t)
    r2 = Rectangle("", h, t)

    self.addFace(b, "base")
    self.attachEdge("base.e2", r1, "e0", prefix="r1", angle=p > 0 and p-180 or -180)
    self.attachEdge("r1.e3", r2, "e1", prefix="r2", angle=90+y)
    self.attachEdge("base.e0", b0, "e2", prefix="mount", angle=p < 0 and 180+p or 180)

if __name__ == "__main__":
    TwoDOF.test()
