from rocolib.api.components import FoldedComponent
from rocolib.api.composables.graph.Face import Face, Rectangle

class Wing(FoldedComponent):
  def define(self):
    self.addParameter("bodylength", 50, paramType="length")
    self.addParameter("wingspan", 100, paramType="length")
    self.addParameter("wingtip", 10, paramType="length")
    self.addParameter("thickness", 10, paramType="length")
    self.addParameter("flip", False, valueType="bool")

    self.addEdgeInterface("base", "bottom.e1", "bodylength")
    self.addEdgeInterface("tip", "bottom.e3", "wingtip")
    self.addEdgeInterface("basetop", "top.e5", 0)
    self.addEdgeInterface("tiptop", "top.e1", 0)

    self.addFaceInterface("bottom", "bottom")

  def assemble(self):
    bodylength = self.p.bodylength
    wingspan = self.p.wingspan
    wingtip = self.p.wingtip
    thickness = self.p.thickness
    flip = self.p.flip

    if flip:
        bodylength, wingtip = wingtip, bodylength

    self.addFace(Face("", (
      (wingspan, 0), (wingspan, bodylength), (0, wingtip), (0,0)
    ), recenter=False), "bottom")

    self.attachEdge("bottom.e2", Face("", (
      (wingspan, 0), (wingspan, wingtip/2.+thickness), (wingspan, wingtip+thickness), (0, bodylength+thickness), (0, bodylength/2.+thickness), (0,0)
    )), "e3", prefix="top", angle = 170)

    self.addTab("bottom.e0", "top.e0", angle= 170, width=thickness)

    if flip:
        self.setEdgeInterface("base", "bottom.e3", "wingtip")
        self.setEdgeInterface("tip", "bottom.e1", "bodylength")
        self.setEdgeInterface("basetop", "top.e1", 0)
        self.setEdgeInterface("tiptop", "top.e5", 0)

if __name__ == "__main__":
  Wing.test()
