import rocolib.utils.numsym as np
from rocolib.utils.transforms import RotateZ, Translate

def prefix(s1, s2):
  if s1 and s2:
    return s1 + "." + s2
  return s1 or s2

def tryImport(module, attribute):
  try:
    mod = __import__(module, fromlist=[attribute])
    obj = getattr(mod, attribute)
    return obj
  except ImportError:
    mod = __import__("rocolib.library." + module, fromlist=[attribute])
    obj = getattr(mod, attribute)
    return obj

def decorateGraph(face, decoration, offset=(0, 0), rotate=False, mode=None):
  try:
    dfaces = decoration.faces
  except AttributeError:
    dfaces = [decoration]

  if mode is None:
    mode = "hole"

  if rotate is False:
    rotate = 0
  elif rotate is True:
    rotate = -90

  for f in dfaces:
    t2d = transformDecorations(face, f.pts2d, offset=offset, rotate=rotate, mode=mode)
    f.transform2D = t2d
    f.addFace(face, np.inv(t2d))

def transformDecorations(face, pts2d, offset=(0,0), rotate=0, flip=False, mode=None):
    a = np.deg2rad(rotate)
    c = np.cos(a)
    s = np.sin(a)

    face.addDecoration(([
        (c*p[0] - s*p[1] + offset[0], s*p[0] + c*p[1] + offset[1])
        for p in pts2d], mode))

    return np.dot(Translate([offset[0], offset[1], 0]), RotateZ(rotate))
